# The Kaggle Book
## Data analysis and machine learning for competitive data science
<B>Code Repository for The Kaggle Book, Published by Packt Publishing</B>


<EM>"Luca and Konradˈs book helps make Kaggle even more accessible. They are both top-ranked users and well-respected members of the Kaggle community. Those who complete this book should expect to be able to engage confidently on Kaggle – and engaging confidently on Kaggle has many rewards."</EM>
<B>— Anthony Goldbloom, Kaggle Founder & CEO</B>

  
<table class="noBorder">
  <tr>
    <td width="50%" height="50%"> 
      <img src="https://github.com/PacktPublishing/Data-Analysis-and-Machine-Learning-with-Kaggle/blob/main/cover.png?raw=true">
    </td>
    <td valign="top">
      <H2>Key Features</H2>
      <ul>
      <li> Learn how Kaggle works and how to make the most of competitions from two expert Kaggle Grandmasters </li>
      <li> Sharpen your modeling skills with ensembling, feature engineering, adversarial validation, AutoML, transfer learning, and techniques for parameter tuning </li>
      <li> Challenge yourself with problems regarding tabular data, vision, natural language as well as simulation and optimization</li>
      <li> Discover tips, tricks, and best practices for getting great results on Kaggle and becoming a better data scientist </li>
      <li> Read interviews with 31 Kaggle Masters and Grandmasters telling about their experience and tips</li>
      </ul>
    </td>
  </tr> 
</table>

 <img src="https://github.com/PacktPublishing/Data-Analysis-and-Machine-Learning-with-Kaggle/blob/main/contributors.jpg?raw=true">

<B><EM>Get a step ahead of your competitors with a concise collection of smart data handling and modeling techniques</EM></B>

## Getting started



You can run these notebooks on cloud platforms like [Kaggle](https://www.kaggle.com/) [Colab](https://colab.research.google.com/) or your local machine. Note that most chapters require a GPU even TPU sometimes to run in a reasonable amount of time, so we recommend one of the cloud platforms as they come pre-installed with CUDA.



### Running on a cloud platform


To run these notebooks on a cloud platform, just click on one of the badges (Colab or Kaggle) in the table below. The code will be reproduced from Github directly onto the choosen platform (you may have to add the necessary data before running it). Alternatively, we also provide links to the fully working original notebook on Kaggle that you can copy and immediately run.

|no| Chapter | Notebook | Colab | Kaggle |
|:--| :-------- | :-------- | :-------: | :-------: |
|05| Competition Tasks and Metrics| [meta_kaggle](https://www.kaggle.com/lucamassaron/meta-kaggle) | [![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_05/meta_kaggle.ipynb) | [![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_05/meta_kaggle.ipynb) |
|06| Designing Good Validation| [adversarial-validation-example](https://www.kaggle.com/code/lucamassaron/adversarial-validation-example) | [![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_06/adversarial-validation-example.ipynb) | [![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_06/adversarial-validation-example.ipynb) |
|07| Modeling for Tabular Competitions | [interesting-eda-tsne-umap](https://www.kaggle.com/lucamassaron/interesting-eda-tsne-umap) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/interesting-eda-tsne-umap.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/interesting-eda-tsne-umap.ipynb)|
||  | [meta-features-and-target-encoding](https://www.kaggle.com/lucamassaron/meta-features-and-target-encoding) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/meta-features-and-target-encoding.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/meta-features-and-target-encoding.ipynb)|
||  | [really-not-missing-at-random](https://www.kaggle.com/lucamassaron/really-not-missing-at-random) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/really-not-missing-at-random.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/really-not-missing-at-random.ipynb)|
||  | [tutorial-feature-selection-with-boruta-shap](https://www.kaggle.com/code/lucamassaron/tutorial-feature-selection-with-boruta-shap) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/tutorial-feature-selection-with-boruta-shap.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_07/tutorial-feature-selection-with-boruta-shap.ipynb)|
|08| Hyperparameter Optimization | [basic-optimization-practices](https://www.kaggle.com/code/lucamassaron/basic-optimization-practices) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/basic-optimization-practices.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/basic-optimization-practices.ipynb)|
||  | [hacking-bayesian-optimization-for-dnns](https://www.kaggle.com/lucamassaron/hacking-bayesian-optimization-for-dnns) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/hacking-bayesian-optimization-for-dnns.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/hacking-bayesian-optimization-for-dnns.ipynb)|
||  | [hacking-bayesian-optimization](https://www.kaggle.com/lucamassaron/hacking-bayesian-optimization) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/hacking-bayesian-optimization.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/hacking-bayesian-optimization.ipynb)|
||  | [kerastuner-for-imdb](https://www.kaggle.com/lucamassaron/kerastuner-for-imdb/) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/kerastuner-for-imdb.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/kerastuner-for-imdb.ipynb)|
||  | [optuna-bayesian-optimization](https://www.kaggle.com/lucamassaron/optuna-bayesian-optimization) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/optuna-bayesian-optimization.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/optuna-bayesian-optimization.ipynb)|
||  | [scikit-optimize-for-lightgbm](https://www.kaggle.com/code/lucamassaron/scikit-optimize-for-lightgbm) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/scikit-optimize-for-lightgbm.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/scikit-optimize-for-lightgbm.ipynb)|
||  | [tutorial-bayesian-optimization-with-lightgbm](https://www.kaggle.com/lucamassaron/tutorial-bayesian-optimization-with-lightgbm) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/tutorial-bayesian-optimization-with-lightgbm.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_08/tutorial-bayesian-optimization-with-lightgbm.ipynb)|
|09| Ensembling with Blending and Stacking Solutions| [ensembling](https://www.kaggle.com/code/lucamassaron/ensembling) |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_09/ensembling.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_09/ensembling.ipynb)|
|10| Modeling for Computer Vision | augmentations-examples |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-augmentations-examples.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-augmentations-examples.ipynb)|
||  | images-classification |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-images-classification.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-images-classification.ipynb)|
||  | prepare-annotations |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-prepare-annotations.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-prepare-annotations.ipynb)|
||  | segmentation-inference |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-segmentation-inference.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-segmentation-inference.ipynb)|
||  | segmentation |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-segmentation.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/ch10-segmentation.ipynb)|
||  | object-detection-yolov5 |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/chap10-object-detection-yolov5.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_10/chap10-object-detection-yolov5.ipynb)|
|11| Modeling for NLP | nlp-augmentations4 |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chap11-nlp-augmentations4.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/***)|
||  | nlp-augmentation1 |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-nlp-augmentation1.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-nlp-augmentation1.ipynb)|
||  | qanswering |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-qanswering.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-qanswering.ipynb)|
||  | sentiment-extraction |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-sentiment-extraction.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_11/chapter11-sentiment-extraction.ipynb)|
|12| Simulation and Optimization Competitions | connectx |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chap12-connectx.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chap12-connectx.ipynb)|
||  | mab-santa |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chapter12-mab-santa.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chapter12-mab-santa.ipynb)|
||  | rps-notebook1 |[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chapter12-rps-notebook1.ipynb)|[![Kaggle](https://kaggle.com/static/images/open-in-kaggle.svg)](https://kaggle.com/kernels/welcome?src=https://github.com/PacktPublishing/The-Kaggle-Book/blob/main/chapter_12/chapter12-rps-notebook1.ipynb)|

## Book Description
Millions of data enthusiasts from around the world compete on Kaggle, the most famous data science competition platform of them all. Participating in Kaggle competitions is a surefire way to improve your data analysis skills, network with the rest of the community, and gain valuable experience to help grow your career.

The first book of its kind, Data Analysis and Machine Learning with Kaggle assembles the techniques and skills you’ll need for success in competitions, data science projects, and beyond. Two masters of Kaggle walk you through modeling strategies you won’t easily find elsewhere, and the tacit knowledge they’ve accumulated along the way. As well as Kaggle-specific tips, you’ll learn more general techniques for approaching tasks based on image data, tabular data, textual data, and reinforcement learning. You’ll design better validation schemes and work more comfortably with different evaluation metrics.

Whether you want to climb the ranks of Kaggle, build some more data science skills, or improve the accuracy of your existing models, this book is for you.

## What you will learn
* Get acquainted with Kaggle and other competition platforms
* Make the most of Kaggle Notebooks, Datasets, and Discussion forums
* Understand different modeling tasks including binary and multi-class classification, object detection, NLP (Natural Language Processing), and time series
* Design good validation schemes, learning about k-fold, probabilistic, and adversarial validation
* Get to grips with evaluation metrics including MSE and its variants, precision and recall, IoU, mean average precision at k, as well as never-before-seen metrics
* Handle simulation and optimization competitions on Kaggle
* Create a portfolio of projects and ideas to get further in your career

## Who This Book Is For
This book is suitable for Kaggle users and data analysts/scientists with at least a basic proficiency in data science topics and Python who are trying to do better in Kaggle competitions and secure jobs with tech giants. At the time of completion of this book, there are 96,190 Kaggle novices (users who have just registered on the website) and 67,666 Kaggle contributors (users who have just filled in their profile) enlisted in Kaggle competitions. This book has been written with all of them in mind and with anyone else wanting to break the ice and start taking part in competitions on Kaggle and learning from them.

## Table of Contents
### Part 1

1.	Introducing Kaggle and Other Data Science Competitions
2.	Organizing Data with Datasets
3.	Working and Learning with Kaggle Notebooks
4.	Leveraging Discussion Forums

### Part 2

5.	Competition Tasks and Metrics
6.	Designing Good Validation
7.	Modeling for Tabular Competitions
8.	Hyperparameter Optimization
9.	Ensembling with Blending and Stacking Solutions
10.	Modeling for Computer Vision
11.	Modeling for NLP
12.	Simulation and Optimization Competitions

### Part 3

13.	Creating Your Portfolio of Projects and Ideas
14.	Finding New Professional Opportunities
